﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;

namespace ThinkGeo.MapSuite.Core
{
    //Displays an arrow (GeoImage) if the value (OneWayValue) is one-way in the column OnWayColumnName.
    //Shows the arrow in the direction of the vertices of the line records.
    class OneWayLineStyle : LineStyle
    {
        private string oneWayColumnName;
        private string oneWayValue;
        private GeoImage geoImage;

        public OneWayLineStyle()
            : this(string.Empty, string.Empty, new GeoImage())
        { }

        public OneWayLineStyle(string oneWayColumnName, string oneWayValue, GeoImage geoImage)
        {
            this.oneWayColumnName = oneWayColumnName;
            this.oneWayValue = oneWayValue;
            this.geoImage = geoImage;
        }

        public string OneWayColumnName
        {
            get { return oneWayColumnName; }
            set { oneWayColumnName = value; }
        }

        public string OneWayValue
        {
            get { return oneWayValue; }
            set { oneWayValue = value; }
        }

        //Image representing the arrow.
        //The arrow of the image should be pointing to the top to have the function GetAngleFromTwoVertices working adequately.
        public GeoImage GeoImage
        {
            get { return geoImage; }
            set { geoImage = value; }
        }

        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, Collection<SimpleCandidate> labelsInThisLayer, Collection<SimpleCandidate> labelsInAllLayers)
        {
            //Loops thru the features to draw the one way feature with the geoImage (arrow)
            foreach (Feature feature in features)
            {
                string currentOneWayValue = Convert.ToString(feature.ColumnValues[oneWayColumnName]);
                if (currentOneWayValue == oneWayValue)
                {
                    MultilineShape multilineShape = (MultilineShape)feature.GetShape();
                    //Gets the LineShape made of the two vertices of the longest segment for displaying the arrow image on it.
                    LineShape lineShape = GetLongestSegment(multilineShape);
                    //Gets the angle based on the two vertices of the longest segment.
                    double angle = GetAngleFromTwoVertices(lineShape.Vertices[0], lineShape.Vertices[1]);
                    //Gets the PointShape in the middle of the longest segment to center the arrow image on.
                    PointShape pointShape = lineShape.GetPointOnALine(StartingPoint.FirstPoint, 50);
                    canvas.DrawWorldImage(geoImage, pointShape.X, pointShape.Y, geoImage.GetWidth(), geoImage.GetHeight(), DrawingLevel.LevelFour, 0, 0, (float)angle);
                }
            }
        }

        private LineShape GetLongestSegment(MultilineShape multiLineShape)
        {
            LineShape resultLineShape = new LineShape();
            double longestDist = 0;

            foreach (LineShape lineShape in multiLineShape.Lines)
            {
                for (int i = 0; i < lineShape.Vertices.Count - 2; i++)
                {
                    PointShape pointShape1 = new PointShape(lineShape.Vertices[i]);
                    PointShape pointShape2 = new PointShape(lineShape.Vertices[i + 1]);
                    double currentDist = pointShape1.GetDistanceTo(pointShape2, GeographyUnit.Meter, DistanceUnit.Meter);
                    if (currentDist > longestDist)
                    {
                        resultLineShape.Vertices.Clear();
                        resultLineShape.Vertices.Add(lineShape.Vertices[i]);
                        resultLineShape.Vertices.Add(lineShape.Vertices[i + 1]);
                        longestDist = currentDist;
                    }
                }
            }
            return resultLineShape;
        }

        private double GetAngleFromTwoVertices(Vertex b, Vertex c)
        {
            double alpha = 0;
            double tangentAlpha = (c.Y - b.Y) / (c.X - b.X);
            double Peta = Math.Atan(tangentAlpha);

            if (c.X > b.X)
            {
                alpha = 90 + (Peta * (180 / Math.PI));
            }
            else if (c.X < b.X)
            {
                alpha = 270 + (Peta * (180 / Math.PI));
            }
            else
            {
                if (c.Y > b.Y) alpha = 0;
                if (c.Y < b.Y) alpha = 180;
            }
            return alpha;
        }

        protected override Collection<string> GetRequiredColumnNamesCore()
        {
            Collection<string> columns = new Collection<string>();

            if (!columns.Contains(oneWayColumnName))
            {
                columns.Add(oneWayColumnName);
            }
            return columns;
        }
    }
}
